//  IoT本格学習キット スマートカー
//  1.Scratch3など遠隔制御用

// ①ライブラリを読み込み
#include <WiFi.h>
#include <ESPAsyncWebServer.h>

// ####################### ②Wi-Fi設定(環境設定) #######################
const char *ssid      = "##### SSID #####";
const char *password  = "### PASSWORD ###";
IPAddress ip(192, 168, 1, 123);  // IPアドレス（本機が利用するIP）
IPAddress gateway(192, 168, 1, 1);  // デフォルトゲートウェイ
IPAddress subnet(255, 255, 255, 0); // サブネットマスク
// ###################################################################

// ③ピン配置
const byte LED_PIN = 15;       // LED緑
const byte R_WHEEL_IN1  = 33;  // 右モータON/OFF
const byte R_WHEEL_IN2  = 25;  // 右モータ回転方向
const byte L_WHEEL_IN1  = 26;  // 左モータON/OFF
const byte L_WHEEL_IN2  = 27;  // 左モータ回転方向
const byte ONPA_TRIG    = 32;  // 超音波トリガー
const byte ONPA_ECHO    = 35;  // 超音波エコー
const byte SERV_PIN     = 12;  // サーボモータ

// ④グローバル変数定義
AsyncWebServer server(80);     // Port80番
bool ledFlag = true;           // LED制御フラグ
const int TURN180_TIME  = 700; // 180度 旋回時間(mSec)
int adjustAngle         = 0;   // Angle補正
int preAdjustAngle      = 0;   // 前回Angle補正
byte preAngel           = -1;  // 現在の角度を保存し、角度変更があるか把握

void setup(void) {
  // ⑤Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑥LED設定
  pinMode ( LED_PIN, OUTPUT );
  pinMode(R_WHEEL_IN1, OUTPUT);
  pinMode(L_WHEEL_IN1, OUTPUT);
  pinMode(R_WHEEL_IN2, OUTPUT);
  pinMode(L_WHEEL_IN2, OUTPUT);
  pinMode(ONPA_TRIG, OUTPUT);
  pinMode(ONPA_ECHO, INPUT);
  ledcSetup(0, 50, 10);       // ch:0, Freq:50Hz, 10bit
  ledcAttachPin(SERV_PIN, 0); // ch:0
  // ⑦無線Wi-Fi接続
  WiFi.config( ip, gateway, subnet );
  WiFi.begin ( ssid, password );
  // ⑧Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑨LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑩１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑪Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑫LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );

  // ⑬WebServerアクセス処理設定[LED制御]
  server.on("/led", HTTP_GET, [](AsyncWebServerRequest *request){
      String setLedVal = request->getParam("o")->value();
      if (setLedVal == "ON") {
        digitalWrite ( LED_PIN, true );
        Serial.println ( "led on" );
      } else {
        digitalWrite ( LED_PIN, false );
        Serial.println ( "led off" );
      }
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑭WebServerアクセス処理設定[超音波センサー角度補正]  
  server.on("/adjusta", HTTP_GET, [](AsyncWebServerRequest *request){
      adjustAngle = request->getParam("n")->value().toInt();
      Serial.println ( "angle adjest:" + String(adjustAngle) );
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑮WebServerアクセス処理設定[超音波センサー角度制御]  
  server.on("/s_angle", HTTP_GET, [](AsyncWebServerRequest *request){
      double angleNum = double(request->getParam("n")->value().toInt());
      Serial.println ( "servo angle:" + String(angleNum) );
      if (preAngel != int(angleNum) || adjustAngle != preAdjustAngle) {
        // パルス幅で角度が決まる（0度:0.5msec[=>26], 180度:2.4msec[=>123]）
        // 90度の場合) 180:1.9(2.4-0.5) = 90:x => x = 1.9*90/180 = 0.95
        // 0.95+0.5 = 1.45msec => 1.45 / 20(10kHz) *1024 => 75 
        // y度の場合) x = ( (1.9*y/180) + 0.5 ) / 20 * 1024
        int angleTime = ceil( ( (1.9*angleNum/180) + 0.5 ) / 20 * 1024 );
        angleTime = angleTime + adjustAngle;
        // 0度:0.5msec[=>26], 180度:2.4msec[=>123]なので、21から128までで制御
        if (angleTime < 21) {
          angleTime = 21;
        } else if (angleTime > 128) {
          angleTime = 128;          
        }
        // サーボモータ回転
        ledcWrite(0, angleTime);
        Serial.println ( "servo angle control:" + request->getParam("n")->value() + ":" + String(angleTime) );
        // 角度を変更する時間を取って返答
        delay ( 1000 );
      } else { // 前へ進んでいる場合（90のままの場合）
        Serial.println ( "servo angle noChange" );
      }
      preAngel = int(angleNum);
      preAdjustAngle = adjustAngle;
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑯WebServerアクセス処理設定[超音波センサー距離測定]  
  server.on("/update_d", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "update distance" );
      // 距離測定と送信メッセージ作成
      String sendMsg = updateSensor();
      // 応答処理
      request->send(200, "application/json", sendMsg);
  });
  // ⑰WebServerアクセス処理設定[Car制御 前進、停止など]  
  server.on("/cargo", HTTP_GET, [](AsyncWebServerRequest *request){
      // リモコン送信
      if ( request->getParam("a")->value() == "STOP" ){
         Serial.println("STOP");
         car_stop();
      } else if ( request->getParam("a")->value() == "BACK" ){
         Serial.println("BACK");
         car_back();
      } else if ( request->getParam("a")->value() == "BACKTURN" ){
        Serial.println("BACKTURN");
        car_backturn();
      } else {
        Serial.println("GO");
        car_go();
      }
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑱WebServerアクセス処理設定[Car回転制御など]  
  server.on("/carturn", HTTP_GET, [](AsyncWebServerRequest *request){
      // リモコン受信設定
      if ( request->getParam("a")->value() == "RIGHT" ){
        Serial.println("RIGHT");
        car_right ( request->getParam("n")->value().toInt() );
      } else {
        Serial.println("LEFT");
        car_left ( request->getParam("n")->value().toInt() );
      }
      // 応答処理
      request->send(200, "text/plain", "OK");
  });
  // ⑲Responseヘッダ追加処理  
  DefaultHeaders::Instance().addHeader("Access-Control-Allow-Origin", "*");
  // ⑳WebServer起動  
  server.begin();
  Serial.println ( "Web server started" );
  // ㉑サーボモータ回転(前向き[90度]へ調整) 
  ledcWrite(0, (76 + adjustAngle));
}

void loop(void){

}
